@@ -36,7 +36,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "07/08/2018"
+__date__ = "28/01/2020"
 __status__ = "production"
 
 
@@ -66,6 +66,10 @@ class _Dectris(Detector):
 class Eiger(_Dectris):
     """
     Eiger detector: generic description containing mask algorithm
+    
+    Nota: 512k modules (514*1030) are made of 2x4 submodules of 256*256 pixels. 
+    Two missing pixels are interpolated at each sub-module boundary which explains 
+    the +2 and the +6 pixels.    
     """
     MODULE_SIZE = (514, 1030)
     MODULE_GAP = (37, 10)
@@ -208,7 +212,7 @@ class Eiger500k(Eiger):
     """
     Eiger 1M detector
     """
-    MAX_SHAPE = (512, 1030)
+    MAX_SHAPE = (514, 1030)
     aliases = ["Eiger 500k"]
 
 
@@ -565,6 +569,13 @@ class PilatusCdTe300kw(PilatusCdTe):
     MAX_SHAPE = (195, 1475)
     aliases = ["Pilatus CdTe 300kw", "Pilatus 300kw CdTe", "Pilatus300kw CdTe", "Pilatus300kwCdTe"]
 
+class PilatusCdTe900kw(PilatusCdTe):
+    """
+    Pilatus CdTe 900k-wide detector
+    """
+    MAX_SHAPE = (195, 4439)
+    aliases = ["Pilatus CdTe 900kw", "Pilatus 900kw CdTe", "Pilatus900kw CdTe", "Pilatus900kwCdTe"]
+
 
 class PilatusCdTe1M(PilatusCdTe):
     """
