@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2012-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2012-2019 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -38,25 +38,26 @@ detector and transform it. It is rather a description of the experimental setup.
 
 from __future__ import division, print_function
 
-__author__ = "Jerome Kieffer"
+__author__ = "Jerome Kieffer" 
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
-__copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "16/05/2019"
+__copyright__ = "European Synchrotron Radiation Facility, Grenoble, France" 
+__date__ = "13/12/2019"    
 __status__ = "production"
 __docformat__ = 'restructuredtext'
 
 import logging
+from math import pi 
 from numpy import radians, degrees, arccos, arctan2, sin, cos, sqrt
 import numpy
 import os
 import threading
 from collections import namedtuple, OrderedDict
-
+from six import PY2
 from . import detectors
 from . import units
 from .utils.decorators import deprecated
-from .utils import crc32
+from .utils import crc32, deg2rad
 from . import utils
 from .io import ponifile
 
@@ -201,18 +202,55 @@ class Geometry(object):
                            f2d["tilt"], f2d["tiltPlanRotation"]))
         return os.linesep.join(lstTxt)
 
-    def check_chi_disc(self, range):
+    def check_chi_disc(self, azimuth_range):
         """Check the position of the :math:`\\chi` discontinuity
 
         :param range: range of chi for the integration
         :return: True if there is a problem
         """
-        lower = range[0]
-        upper = range[-1]
-        disc = numpy.pi if self.chiDiscAtPi else 0
-        if (lower < disc) and (upper > disc):
-            logger.warning("Chi discontinuity in azimuthal range ! disc=%s in [%s, %s]", disc, lower, upper)
-            return 1
+        lower, upper = azimuth_range
+        error_msg = "Azimuthal range issue: Range [%s, %s] not in valid region %s in radians: Expect %s results !"
+        if self.chiDiscAtPi:
+            txt_range = "[-pi; pi[" if PY2 else "[-π; π[" 
+            lower_bound = -pi
+            upper_bound = pi 
+        else:
+            txt_range = "[0; 2pi[" if PY2 else "[-0; 2π["
+            lower_bound = 0
+            upper_bound = 2*pi 
+
+        if lower<lower_bound:
+            if upper<lower_bound:
+                logger.warning(error_msg, lower, upper, txt_range, "empty")
+            else:
+                logger.warning(error_msg, lower, upper, txt_range,"partial")
+            return True
+        elif lower>upper_bound:
+            logger.warning(error_msg, lower, upper, txt_range, "empty")
+            return True
+        else: 
+            if upper>upper_bound:
+                logger.warning(error_msg, lower, upper, txt_range, "partial")
+                return True
+        return False
+
+    def normalize_azimuth_range(self, azimuth_range):
+        """Convert the azimuth range from degrees to radians 
+        
+        This method takes care of the position of the discontinuity and adapts the range accordingly!
+        
+        :param azimuth_range: 2-tuple of float in degrees
+        :return: 2-tuple of float in radians in a range such to avoid the discontinuity
+        """
+        if azimuth_range is None:
+            return
+        azimuth_range = tuple(deg2rad(azimuth_range[i], self.chiDiscAtPi) for i in (0, -1))
+        if azimuth_range[1] <= azimuth_range[0]:
+            azimuth_range = (azimuth_range[0], azimuth_range[1] + 2 * pi)
+            self.check_chi_disc(azimuth_range)
+        return azimuth_range
+
+        
 
     def _calc_cartesian_positions(self, d1, d2, poni1=None, poni2=None):
         """
@@ -633,7 +671,7 @@ class Geometry(object):
         :param use_cython: set to False to use the slower Python path (for tests)
         :param scale: set to False for returning the internal representation
                       (S.I. often) which is faster
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
             - dim3[0]: radial angle 2th, q, r...
             - dim3[1]: azimuthal angle chi
         """
@@ -731,7 +769,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
            * dim3[0]: radial angle 2th
            * dim3[1]: azimuthal angle chi
         """
@@ -745,7 +783,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are (scattering vector q, azimuthal angle chi)
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are (scattering vector q, azimuthal angle chi)
         """
         return self.corner_array(shape, unit=units.Q, use_cython=False, scale=False)
 
@@ -757,7 +795,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are (radial distance, azimuthal angle chi)
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are (radial distance, azimuthal angle chi)
         """
         return self.corner_array(shape, unit=units.R, use_cython=False, scale=False)
 
@@ -769,7 +807,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are (reciprocal spacing squared, azimuthal angle chi)
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are (reciprocal spacing squared, azimuthal angle chi)
         """
         return self.corner_array(shape, unit=units.RecD2_NM, scale=False)
 
@@ -783,7 +821,7 @@ class Geometry(object):
         :param unit: string like "2th_deg" or an instance of pyFAI.units.Unit
         :param scale: set to False for returning the internal representation
                 (S.I. often) which is faster
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
             - dim3[0]: radial angle 2th, q, r...
             - dim3[1]: azimuthal angle chi
         """
@@ -825,7 +863,7 @@ class Geometry(object):
         :param unit: string like "2th_deg" or an instance of pyFAI.units.Unit
         :param scale: set to False for returning the internal representation
                 (S.I. often) which is faster
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
 
             - dim3[0]: radial angle 2th, q, r...
             - dim3[1]: azimuthal angle chi
@@ -1454,10 +1492,10 @@ class Geometry(object):
             tilt_y -0.013173
             tilt_z 0.002378
             wavelength 0.154
-            y-center 1016.328171
-            y-size 48.0815
-            z-center 984.924425
-            z-size 46.77648
+            y_center 1016.328171
+            y_size 48.0815
+            z_center 984.924425
+            z_size 46.77648
         """
         f2d = self.getFit2D()
         distance = f2d.get("directDist", 0) * 1e3  # mm -> µm
@@ -1479,17 +1517,17 @@ class Geometry(object):
         if self._wavelength:
             out["wavelength"] = self.wavelength * 1e9  # nm
         if y_center:
-            out["y-center"] = y_center
-        out["y-size"] = self.detector.pixel2 * 1e6  # µm
+            out["y_center"] = y_center
+        out["y_size"] = self.detector.pixel2 * 1e6  # µm
         if z_center:
-            out["z-center"] = z_center
-        out["z-size"] = self.detector.pixel1 * 1e6  # µm
+            out["z_center"] = z_center
+        out["z_size"] = self.detector.pixel1 * 1e6  # µm
         return out
 
     def setImageD11(self, param):
         """Set the geometry from the parameter set which contains distance, 
-        o11, o12, o21, o22, tilt_x, tilt_y tilt_z, wavelength, y-center, y-size, 
-        z-center and z-size. 
+        o11, o12, o21, o22, tilt_x, tilt_y tilt_z, wavelength, y_center, y_size, 
+        z_center and z_size. 
         Please refer to the documentation in doc/source/geometry_conversion.rst
         for the orientation and units of those values.
         
@@ -1513,10 +1551,10 @@ class Geometry(object):
         self.rot1 = -param.get("tilt_z", 0.0)
         distance = param.get("distance", 0.0) * 1e-6  # ->m
         self.dist = distance * cos(self.rot2) * cos(self.rot1)
-        pixel_v = param.get("z-size", 0.0) * 1e-6
-        pixel_h = param.get("y-size", 0.0) * 1e-6
-        self.poni1 = -distance * sin(self.rot2) + pixel_v * param.get("z-center", 0.0)
-        self.poni2 = +distance * cos(self.rot2) * sin(self.rot1) + pixel_h * param.get("y-center", 0.0)
+        pixel_v = param.get("z_size", 0.0) * 1e-6
+        pixel_h = param.get("y_size", 0.0) * 1e-6
+        self.poni1 = -distance * sin(self.rot2) + pixel_v * param.get("z_center", 0.0)
+        self.poni2 = +distance * cos(self.rot2) * sin(self.rot1) + pixel_h * param.get("y_center", 0.0)
         self.detector = detectors.Detector(pixel1=pixel_v, pixel2=pixel_h)
         wl = param.get("wavelength")
         if wl:
