@@ -33,7 +33,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "18/10/2018"
+__date__ = "24/01/2020"
 __status__ = "development"
 __docformat__ = 'restructuredtext'
 
@@ -42,35 +42,36 @@ import time
 import json
 import threading
 import logging
+
 from silx.gui import qt
+from silx.gui import icons
 
 from .matplotlib import pyplot
 from ..utils import int_, str_, get_ui_file
 from ..units import to_unit
-from .integrate_widget import AIWidget
+from .widgets.WorkerConfigurator import WorkerConfigurator
 from .. import worker
+from ..io.integration_config import ConfigurationReader
 from ..diffmap import DiffMap
 from .utils.tree import ListDataSet, DataSet
 
 logger = logging.getLogger(__name__)
 
 
-class IntegrateWidget(qt.QDialog):
+class IntegrateDialog(qt.QDialog):
     def __init__(self, parent=None):
         qt.QDialog.__init__(self)
-        self.widget = AIWidget()
-        self.layout = qt.QGridLayout(self)
-        self.layout.addWidget(self.widget)
-        self.widget.okButton.clicked.disconnect()
-        self.widget.do_2D.setEnabled(False)
-        self.widget.cancelButton.clicked.disconnect()
-        self.widget.okButton.clicked.connect(self.accept)
-        self.widget.cancelButton.clicked.connect(self.reject)
+        self.widget = WorkerConfigurator(self)
+        self.widget.set1dIntegrationOnly(True)
 
-    def get_config(self):
-        res = self.widget.dump()
-        res["method"] = self.widget.get_method()
-        return res
+        layout = qt.QVBoxLayout(self)
+        layout.addWidget(self.widget)
+        buttons = qt.QDialogButtonBox(self)
+        buttons.setStandardButtons(qt.QDialogButtonBox.Cancel | qt.QDialogButtonBox.Ok)
+        layout.addWidget(buttons)
+
+        buttons.accepted.connect(self.accept)
+        buttons.rejected.connect(self.reject)
 
 
 class TreeModel(qt.QAbstractItemModel):
@@ -85,10 +86,8 @@ class TreeModel(qt.QAbstractItemModel):
         new_labels = [i.label for i in new_root.children]
         old_lables = [i.label for i in self._root_item.children]
         if new_labels == old_lables:
-            print("update labels")
             self._root_item.update(new_root)
         else:
-            print("replace labels")
             self._root_item.children = []
             for child in new_root.children:
                 self._root_item.add_child(child)
@@ -135,7 +134,6 @@ class TreeModel(qt.QAbstractItemModel):
     def parent(self, midx):
         item = midx.internalPointer()
         if (item is None) or (item is self._root_item):
-            print(midx, midx.row(), midx.column())
             return  # QtCore.QModelIndex()
         pitem = item.parent
         if pitem is self._root_item:
@@ -161,6 +159,10 @@ class DiffMapWidget(qt.QWidget):
         except AttributeError as _error:
             logger.error("I looks like your installation suffers from this bug: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=697348")
             raise RuntimeError("Please upgrade your installation of PyQt (or apply the patch)")
+
+        pyfaiIcon = icons.getQIcon("pyfai:gui/images/icon")
+        self.setWindowIcon(pyfaiIcon)
+
         self.aborted = False
         self.progressBar.setValue(0)
         self.list_model = TreeModel(self, self.list_dataset.as_tree())
@@ -271,7 +273,6 @@ class DiffMapWidget(qt.QWidget):
     def clear_selection(self, *args, **kwargs):
         """called to remove selected files from the list
         """
-        print(self.listFiles.selectedIndexes())
         logger.warning("remove all files for now !! not yet implemented")
         self.list_dataset.empty()
         self.list_model.update(self.list_dataset.as_tree())
@@ -280,15 +281,20 @@ class DiffMapWidget(qt.QWidget):
         """
         """
         logger.info("in configure_diffraction")
-        iw = IntegrateWidget(self)
+        iw = IntegrateDialog(self)
         if self.integration_config:
-            iw.widget.set_config(self.integration_config)
-        res = iw.exec_()
-        if res == qt.QDialog.Accepted:
-            iw.widget.input_data = [i.path for i in self.list_dataset]
-            self.integration_config = iw.get_config()
-        print(json.dumps(self.integration_config, indent=2))
-
+            iw.widget.setConfig(self.integration_config)
+        while True:
+            res = iw.exec_()
+            if res == qt.QDialog.Accepted:
+                self.integration_config = iw.widget.getConfig()
+                if self.integration_config.get("nbpt_rad"):
+                    break
+                else:
+                    qt.QMessageBox.about(self, "Unconsistent configuration", "Some essential parameters are missing ... Did you set the radial number of points ?")
+            else:
+                break
+        
     def configure_output(self, *args, **kwargs):
         """
         called when clicking on "outputFileSelector"
@@ -437,15 +443,17 @@ class DiffMapWidget(qt.QWidget):
         with self.processing_sem:
             config = self.dump()
             config_ai = config.get("ai", {})
+            config_ai = config_ai.copy()
+            if "nbpt_rad" not in config_ai:
+                raise RuntimeError("The number of radial points is mandatory !")
+
             diffmap = DiffMap(npt_fast=config.get("fast_motor_points", 1),
                               npt_slow=config.get("slow_motor_points", 1),
                               npt_rad=config_ai.get("nbpt_rad", 1000),
                               npt_azim=config_ai.get("nbpt_azim", 1) if config_ai.get("do_2D") else None)
             diffmap.inputfiles = [i.path for i in self.list_dataset]  # in case generic detector without shape
-            print(config_ai)
-            diffmap.ai = worker.make_ai(config_ai)
-            diffmap.method = config_ai.get("method", "csr")
-            diffmap.unit = to_unit(config_ai.get("unit", "2th_deg"))
+            diffmap.worker = worker.Worker()
+            diffmap.worker.set_config(config_ai, consume_keys=False)
             diffmap.hdf5 = config.get("output_file", "unamed.h5")
             self.radial_data = diffmap.init_ai()
             self.data_h5 = diffmap.dataset
@@ -456,11 +464,11 @@ class DiffMapWidget(qt.QWidget):
                     logger.warning("Aborted by user")
                     self.progressbarChanged.emit(0, 0)
                     if diffmap.nxs:
-                        self.data_np = diffmap.dataset.value
+                        self.data_np = diffmap.dataset[()]
                         diffmap.nxs.close()
                     return
             if diffmap.nxs:
-                self.data_np = diffmap.dataset.value
+                self.data_np = diffmap.dataset[()]
                 diffmap.nxs.close()
         logger.warning("Processing finished in %.3fs", time.time() - t0)
         self.progressbarChanged.emit(len(self.list_dataset), 0)
@@ -492,11 +500,17 @@ class DiffMapWidget(qt.QWidget):
         """
         if idx_file >= 0:
             self.progressBar.setValue(idx_file)
-        if self.update_sem._Semaphore__value < 1:
+
+        # Check if there is a free semaphore without blocking
+        if self.update_sem.acquire(blocking=False):
+            self.update_sem.release()
+        else:
+            # It's full
             return
+
         with self.update_sem:
             try:
-                data = self.data_h5.value
+                data = self.data_h5[()]
             except ValueError:
                 data = self.data_np
             if self.radial_data is None:
