@@ -3,7 +3,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2019 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -45,7 +45,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "01/03/2019"
+__date__ = "18/10/2019"
 __status__ = "production"
 __docformat__ = 'restructuredtext'
 
@@ -84,58 +84,7 @@ except ImportError:
     logger.error("fabio module missing")
 
 
-def get_isotime(forceTime=None):
-    """
-    :param forceTime: enforce a given time (current by default)
-    :type forceTime: float
-    :return: the current time as an ISO8601 string
-    :rtype: string
-    """
-    if forceTime is None:
-        forceTime = time.time()
-    localtime = time.localtime(forceTime)
-    gmtime = time.gmtime(forceTime)
-    tz_h = localtime.tm_hour - gmtime.tm_hour
-    tz_m = localtime.tm_min - gmtime.tm_min
-    return "%s%+03i:%02i" % (time.strftime("%Y-%m-%dT%H:%M:%S", localtime), tz_h, tz_m)
-
-
-def from_isotime(text, use_tz=False):
-    """
-    :param text: string representing the time is iso format
-    """
-    if len(text) == 1:
-        # just in case someone sets as a list
-        text = text[0]
-    try:
-        text = text.decode("ascii")
-    except (UnicodeError, AttributeError):
-        text = str(text)
-    if len(text) < 19:
-        logger.warning("Not a iso-time string: %s", text)
-        return
-    base = text[:19]
-    if use_tz and len(text) == 25:
-        sgn = 1 if text[:19] == "+" else -1
-        tz = 60 * (60 * int(text[20:22]) + int(text[23:25])) * sgn
-    else:
-        tz = 0
-    return time.mktime(time.strptime(base, "%Y-%m-%dT%H:%M:%S")) + tz
-
-
-def is_hdf5(filename):
-    """
-    Check if a file is actually a HDF5 file
-
-    :param filename: this file has better to exist
-    """
-    signature = [137, 72, 68, 70, 13, 10, 26, 10]
-    if not os.path.exists(filename):
-        raise IOError("No such file %s" % (filename))
-    with open(filename, "rb") as f:
-        raw = f.read(8)
-    sig = [ord(i) for i in raw] if sys.version_info[0] < 3 else [int(i) for i in raw]
-    return sig == signature
+from .nexus import get_isotime, from_isotime, is_hdf5, Nexus
 
 
 class Writer(object):
@@ -305,7 +254,7 @@ class HDF5Writer(Writer):
             if mode == self.MODE_DELETE:
                 logger.error("File can't be read. File %s deleted.", self.filename)
                 os.unlink(self.filename)
-                self.hdf5 = h5py.File(self.filename)
+                self.hdf5 = h5py.File(self.filename, mode="w")
             else:
                 raise
 
@@ -318,7 +267,7 @@ class HDF5Writer(Writer):
                 self.hdf5.close()
                 logger.warning("File already contains an entry. File %s deleted.", self.filename)
                 os.unlink(self.filename)
-                self.hdf5 = h5py.File(self.filename)
+                self.hdf5 = h5py.File(self.filename, mode="w")
             elif mode == self.MODE_APPEND:
                 name = self._find_unused_name(self.hpath)
             elif mode == self.MODE_OVERWRITE:
@@ -965,247 +914,3 @@ class FabioWriter(Writer):
                 numpy.savetxt(f, data)
 
 
-class Nexus(object):
-    """
-    Writer class to handle Nexus/HDF5 data
-
-    Manages:
-
-    - entry
-
-        - pyFAI-subentry
-
-            - detector
-
-    TODO: make it thread-safe !!!
-    """
-
-    def __init__(self, filename, mode="r"):
-        """
-        Constructor
-
-        :param filename: name of the hdf5 file containing the nexus
-        :param mode: can be r or a
-        """
-        self.filename = os.path.abspath(filename)
-        self.mode = mode
-        if not h5py:
-            logger.error("h5py module missing: NeXus not supported")
-            raise RuntimeError("H5py module is missing")
-        pre_existing = os.path.exists(self.filename)
-        if pre_existing and self.mode == "r":
-            self.h5 = h5py.File(self.filename, mode=self.mode)
-        else:
-            self.h5 = h5py.File(self.filename)
-        self.to_close = []
-
-        if not pre_existing:
-            self.h5.attrs["NX_class"] = "NXroot"
-            self.h5.attrs["file_time"] = get_isotime()
-            self.h5.attrs["file_name"] = self.filename
-            self.h5.attrs["HDF5_Version"] = h5py.version.hdf5_version
-            self.h5.attrs["creator"] = self.__class__.__name__
-
-    def close(self):
-        """
-        close the filename and update all entries
-        """
-        if self.mode != "r":
-            end_time = get_isotime()
-            for entry in self.to_close:
-                entry["end_time"] = end_time
-            self.h5.attrs["file_update_time"] = get_isotime()
-        self.h5.close()
-
-    # Context manager for "with" statement compatibility
-    def __enter__(self, *arg, **kwarg):
-        return self
-
-    def __exit__(self, *arg, **kwarg):
-        self.close()
-
-    def get_entry(self, name):
-        """
-        Retrieves an entry from its name
-
-        :param name: name of the entry to retrieve
-        :return: HDF5 group of NXclass == NXentry
-        """
-        for grp_name in self.h5:
-            if grp_name == name:
-                grp = self.h5[grp_name]
-                if isinstance(grp, h5py.Group) and \
-                   ("start_time" in grp) and  \
-                   self.get_attr(grp, "NX_class") == "NXentry":
-                        return grp
-
-    def get_entries(self):
-        """
-        retrieves all entry sorted the latest first.
-
-        :return: list of HDF5 groups
-        """
-        entries = [(grp, from_isotime(self.h5[grp + "/start_time"].value))
-                   for grp in self.h5
-                   if isinstance(self.h5[grp], h5py.Group) and
-                   ("start_time" in self.h5[grp]) and
-                   self.get_attr(self.h5[grp], "NX_class") == "NXentry"]
-        entries.sort(key=lambda a: a[1], reverse=True)  # sort entries in decreasing time
-        return [self.h5[i[0]] for i in entries]
-
-    def find_detector(self, all=False):
-        """
-        Tries to find a detector within a NeXus file, takes the first compatible detector
-
-        :param all: return all detectors found as a list
-        """
-        result = []
-        for entry in self.get_entries():
-            for instrument in self.get_class(entry, "NXsubentry") + self.get_class(entry, "NXinstrument"):
-                for detector in self.get_class(instrument, "NXdetector"):
-                    if all:
-                        result.append(detector)
-                    else:
-                        return detector
-        return result
-
-    def new_entry(self, entry="entry", program_name="pyFAI",
-                  title="description of experiment",
-                  force_time=None, force_name=False):
-        """
-        Create a new entry
-
-        :param entry: name of the entry
-        :param program_name: value of the field as string
-        :param title: value of the field as string
-        :param force_time: enforce the start_time (as string!)
-        :param force_name: force the entry name as such, without numerical suffix.
-        :return: the corresponding HDF5 group
-        """
-
-        if not force_name:
-            nb_entries = len(self.get_entries())
-            entry = "%s_%04i" % (entry, nb_entries)
-        entry_grp = self.h5.require_group(entry)
-        self.h5.attrs["default"] = entry
-        entry_grp.attrs["NX_class"] = numpy.string_("NXentry")
-        entry_grp["title"] = numpy.string_(title)
-        entry_grp["program_name"] = numpy.string_(program_name)
-        if force_time:
-            entry_grp["start_time"] = numpy.string_(force_time)
-        else:
-            entry_grp["start_time"] = numpy.string_(get_isotime())
-        self.to_close.append(entry_grp)
-        return entry_grp
-
-    def new_instrument(self, entry="entry", instrument_name="id00",):
-        """
-        Create an instrument in an entry or create both the entry and the instrument if
-        """
-        if not isinstance(entry, h5py.Group):
-            entry = self.new_entry(entry)
-        return self.new_class(entry, instrument_name, "NXinstrument")
-#        howto external link
-        # myfile['ext link'] = h5py.ExternalLink("otherfile.hdf5", "/path/to/resource")
-
-    def new_class(self, grp, name, class_type="NXcollection"):
-        """
-        create a new sub-group with  type class_type
-        :param grp: parent group
-        :param name: name of the sub-group
-        :param class_type: NeXus class name
-        :return: subgroup created
-        """
-        sub = grp.require_group(name)
-        sub.attrs["NX_class"] = numpy.string_(class_type)
-        return sub
-
-    def new_detector(self, name="detector", entry="entry", subentry="pyFAI"):
-        """
-        Create a new entry/pyFAI/Detector
-
-        :param detector: name of the detector
-        :param entry: name of the entry
-        :param subentry: all pyFAI description of detectors should be in a pyFAI sub-entry
-        """
-        entry_grp = self.new_entry(entry)
-        pyFAI_grp = self.new_class(entry_grp, subentry, "NXsubentry")
-        pyFAI_grp["definition_local"] = numpy.string_("pyFAI")
-        pyFAI_grp["definition_local"].attrs["version"] = numpy.string_(version)
-        det_grp = self.new_class(pyFAI_grp, name, "NXdetector")
-        return det_grp
-
-    def get_class(self, grp, class_type="NXcollection"):
-        """
-        return all sub-groups of the given type within a group
-
-        :param grp: HDF5 group
-        :param class_type: name of the NeXus class
-        """
-        coll = [grp[name] for name in grp
-                if isinstance(grp[name], h5py.Group) and
-                self.get_attr(grp[name], "NX_class") == class_type]
-        return coll
-
-    def get_data(self, grp, class_type="NXdata"):
-        """
-        return all dataset of the the NeXus class NXdata
-
-        :param grp: HDF5 group
-        :param class_type: name of the NeXus class
-        """
-        coll = [grp[name] for name in grp
-                if isinstance(grp[name], h5py.Dataset) and
-                self.get_attr(grp[name], "NX_class") == class_type]
-        return coll
-
-    def deep_copy(self, name, obj, where="/", toplevel=None, excluded=None, overwrite=False):
-        """
-        perform a deep copy:
-        create a "name" entry in self containing a copy of the object
-
-        :param where: path to the toplevel object (i.e. root)
-        :param  toplevel: firectly the top level Group
-        :param excluded: list of keys to be excluded
-        :param overwrite: replace content if already existing
-        """
-        if (excluded is not None) and (name in excluded):
-            return
-        if not toplevel:
-            toplevel = self.h5[where]
-        if isinstance(obj, h5py.Group):
-            if name not in toplevel:
-                grp = toplevel.require_group(name)
-                for k, v in obj.attrs.items():
-                        grp.attrs[k] = v
-        elif isinstance(obj, h5py.Dataset):
-            if name in toplevel:
-                if overwrite:
-                    del toplevel[name]
-                    logger.warning("Overwriting %s in %s", toplevel[name].name, self.filename)
-                else:
-                    logger.warning("Not overwriting %s in %s", toplevel[name].name, self.filename)
-                    return
-            toplevel[name] = obj.value
-            for k, v in obj.attrs.items():
-                toplevel[name].attrs[k] = v
-
-    @classmethod
-    def get_attr(cls, dset, name, default=None):
-        """Return the attribute of the dataset
-
-        Handles the ascii -> unicode issue in python3 #275
-
-        :param dset: a HDF5 dataset (or a group)
-        :param name: name of the attribute
-        :param default: default value to be returned
-        :return: attribute value decoded in python3 or default
-        """
-        dec = default
-        if name in dset.attrs:
-            raw = dset.attrs[name]
-            if (sys.version_info[0] > 2) and ("decode" in dir(raw)):
-                dec = raw.decode()
-            else:
-                dec = raw
-        return dec
