@@ -85,126 +85,154 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "04/05/2018"
+__date__ = "24/01/2020"
 __status__ = "development"
 
 import threading
 import os.path
 import logging
 import json
+import numpy
 
 logger = logging.getLogger(__name__)
 
-import numpy
-import fabio
-from .detectors import detector_factory
+from .third_party import six
+from . import average
+from . import method_registry
 from .azimuthalIntegrator import AzimuthalIntegrator
 from .distortion import Distortion
 from . import units
+from .io import integration_config
+import pyFAI.io.image
+from .engines.preproc import preproc as preproc_numpy
 try:
     from .ext.preproc import preproc
-    USE_CYTHON = True
 except ImportError as err:
-    logger.warning("Unable to import preproc: %s", err)
-    preproc = None
+    logger.warning("Unable to import Cython version of preproc: %s", err)
+    preproc = preproc_numpy
     USE_CYTHON = False
 else:
     USE_CYTHON = True
 
 
-def make_ai(config):
-    """Create an Azimuthal integrator from the configuration
-    stand alone function !
+def make_ai(config, consume_keys=False):
+    """Create an Azimuthal integrator from the configuration.
 
-    :param config: dict with all parameters
-    :return: configured (but uninitialized) AzimuthalIntgrator
+    :param config: Key-value dictionary with all parameters
+    :param bool consume_keys: If true the keys from the dictionary will be
+        consumed when used.
+    :return: A configured (but uninitialized) :class:`AzimuthalIntgrator`.
     """
-    poni = config.get("poni")
-    if poni and os.path.isfile(poni):
-        ai = AzimuthalIntegrator.sload(poni)
-    else:
-        ai = AzimuthalIntegrator()
+    config = integration_config.normalize(config, inplace=consume_keys)
+    ai = AzimuthalIntegrator()
+    _init_ai(ai, config, consume_keys)
+    return ai
 
-    detector = config.get("detector", None)
-    if detector:
-        ai.detector = detector_factory(detector)
 
-    wavelength = config.get("wavelength", 0)
-    if wavelength:
-        if wavelength <= 0 or wavelength > 1e-6:
-            logger.warning("Wavelength is in meter ... unlikely value %s", wavelength)
-        ai.wavelength = wavelength
+def _init_ai(ai, config, consume_keys=False, read_maps=True):
+    """Initialize an :class:`AzimuthalIntegrator` from a configuration.
 
-    splinefile = config.get("splineFile")
-    if splinefile and os.path.isfile(splinefile):
-        ai.detector.splineFile = splinefile
+    :param AzimuthalIntegrator ai: An :class:`AzimuthalIntegrator`.
+    :param config: Key-value dictionary with all parameters
+    :param bool consume_keys: If true the keys from the dictionary will be
+        consumed when used.
+    :param bool read_maps: If true mask, flat, dark will be read.
+    :return: A configured (but uninitialized) :class:`AzimuthalIntgrator`.
+    """
+    if not consume_keys:
+        config = dict(config)
 
-    for key in ("pixel1", "pixel2", "dist", "poni1", "poni2", "rot1", "rot2", "rot3"):
-        value = config.get(key)
+    # Geometry
+    for key in ("dist", "poni1", "poni2", "rot1", "rot2", "rot3"):
+        value = config.pop(key, None)
         if value is not None:
             ai.__setattr__(key, value)
-    if config.get("chi_discontinuity_at_0"):
+    wavelength = config.pop("wavelength", None)
+    if wavelength:
+        if wavelength <= 0 or wavelength > 1e-6:
+            logger.warning("Wavelength is in meter... unlikely value %s", wavelength)
+        ai.wavelength = wavelength
+
+    # Detector
+    reader = integration_config.ConfigurationReader(config)
+    detector = reader.pop_detector()
+    if detector is not None:
+        ai.detector = detector
+
+    value = config.pop("chi_discontinuity_at_0", False)
+    if value:
         ai.setChiDiscAtZero()
+    else:
+        ai.setChiDiscAtPi()
 
-    mask_file = config.get("mask_file")
-    if mask_file and config.get("do_mask"):
-        if os.path.exists(mask_file):
+    if read_maps:
+        filename = config.pop("mask_file", "")
+        apply_process = config.pop("do_mask", True)
+        if filename and apply_process:
             try:
-                mask = fabio.open(mask_file).data
+                data = pyFAI.io.image.read_image_data(filename)
             except Exception as error:
-                logger.error("Unable to load mask file %s, error %s", mask_file, error)
+                logger.error("Unable to load mask file %s, error %s", filename, error)
             else:
-                ai.mask = mask
+                ai.detector.mask = data
 
-    dark_files = [i.strip() for i in config.get("dark_current", "").split(",")
-                  if os.path.isfile(i.strip())]
-    if dark_files and config.get("do_dark"):
-        ai.set_darkfiles(dark_files)
-
-    flat_files = [i.strip() for i in config.get("flat_field", "").split(",")
-                  if os.path.isfile(i.strip())]
-    if flat_files and config.get("do_flat"):
-        ai.set_flatfiles(flat_files)
+        filename = config.pop("dark_current", "")
+        apply_process = config.pop("do_dark", True)
+        if filename and apply_process:
+            ai.detector.set_darkcurrent(_reduce_images(_normalize_filenames(filename)))
 
+        filename = config.pop("flat_field", "")
+        apply_process = config.pop("do_flat", True)
+        if filename and apply_process:
+            ai.detector.set_flatfield(_reduce_images(_normalize_filenames(filename)))
     return ai
 
 
+def _normalize_filenames(filenames):
+    """Returns a list of strings from a string or a list of strings.
+
+    :rtype: List[str]
+    """
+    if filenames is None or filenames == "":
+        return []
+    if isinstance(filenames, list):
+        return filenames
+    if isinstance(filenames, six.string_types):
+        # It's a single filename
+        return [filenames]
+    raise TypeError("Unsupported type %s for a list of filenames" % type(filenames))
+
+
+def _reduce_images(filenames, method="mean"):
+    """
+    Reduce a set of filenames using a reduction method
+
+    :param List[str] filenames: List of files used to compute the data
+    :param str method: method used to compute the dark, "mean" or "median"
+    """
+    if len(filenames) == 0:
+        return None
+    if len(filenames) == 1:
+        return pyFAI.io.image.read_image_data(filenames[0]).astype(numpy.float32)
+    else:
+        return average.average_images(filenames, filter_=method, fformat=None, threshold=0)
+
+
 class Worker(object):
     def __init__(self, azimuthalIntegrator=None,
                  shapeIn=(2048, 2048), shapeOut=(360, 500),
-                 unit="r_mm", dummy=None, delta_dummy=None,
-                 azimuthalIntgrator=None):
+                 unit="r_mm", dummy=None, delta_dummy=None):
         """
         :param AzimuthalIntegrator azimuthalIntegrator: An AzimuthalIntegrator instance
-        :param AzimuthalIntegrator azimuthalIntgrator: An AzimuthalIntegrator instance (deprecated)
         :param shapeIn: image size in input
         :param shapeOut: Integrated size: can be (1,2000) for 1D integration
         :param unit: can be "2th_deg, r_mm or q_nm^-1 ...
         """
-        # TODO remove it in few month (added on 2016-08-04)
-        if azimuthalIntgrator is not None:
-            logger.warning("'Worker(azimuthalIntgrator=...)' parameter is deprecated cause it contains a typo. Please use 'azimuthalIntegrator='")
-            azimuthalIntegrator = azimuthalIntgrator
-
         self._sem = threading.Semaphore()
         if azimuthalIntegrator is None:
             self.ai = AzimuthalIntegrator()
         else:
             self.ai = azimuthalIntegrator
-#        self.config = {}
-#        self.config_file = "azimInt.json"
-#        self.nbpt_azim = 0
-#        if type(config) == dict:
-#            self.config = config
-#        elif type(config) in types.StringTypes:
-#            if os.path.isfile(config):
-#                with open(config, "r") as f:
-#                    self.config = json.load(f)
-#                self.config_file(config)
-#            else:
-#                self.config = json.loads(config)
-#        if self.config:
-#            self.configure()
         self._normalization_factor = None  # Value of the monitor: divides the intensity by this value for normalization
         self.nbpt_azim, self.nbpt_rad = shapeOut
         self._unit = units.to_unit(unit)
@@ -260,7 +288,6 @@ class Worker(object):
                 if self.needs_reset:
                     self.ai.reset()
                     self.needs_reset = False
-        # print self.__repr__()
 
     def reconfig(self, shape=(2048, 2048), sync=False):
         """
@@ -273,7 +300,7 @@ class Worker(object):
         self.ai.reset()
         self.warmup(sync)
 
-    def process(self, data, normalization_factor=1.0, writer=None, metadata=None):
+    def process(self, data, variance=None, normalization_factor=1.0, writer=None, metadata=None):
         """
         Process a frame
         #TODO:
@@ -291,10 +318,10 @@ class Worker(object):
                  "method": self.method,
                  "polarization_factor": self.polarization_factor,
                  # "filename": None,
-                 "safe": self.safe,
                  "data": data,
                  "correctSolidAngle": self.correct_solid_angle,
-                 "safe": self.safe
+                 "safe": self.safe,
+                 "variance": variance
                  }
 
         if metadata is not None:
@@ -318,10 +345,7 @@ class Worker(object):
             #        kwarg["filename"] += self.extension
             #    else:
             #        kwarg["filename"] += ".xy"
-        if self.do_poisson:
-            kwarg["error_model"] = "poisson"
-        else:
-            kwarg["error_model"] = None
+        kwarg["error_model"] = self.error_model
 
         if self.radial_range is not None:
             kwarg["radial_range"] = self.radial_range
@@ -329,12 +353,15 @@ class Worker(object):
         if self.azimuth_range is not None:
             kwarg["azimuth_range"] = self.azimuth_range
 
+        error = None
         try:
             if self.do_2D():
                 integrated_result = self.ai.integrate2d(**kwarg)
                 self.radial = integrated_result.radial
                 self.azimuthal = integrated_result.azimuthal
                 result = integrated_result.intensity
+                if variance is not None:
+                    error = integrated_result.sigma
             else:
                 integrated_result = self.ai.integrate1d(**kwarg)
                 self.radial = integrated_result.radial
@@ -342,6 +369,7 @@ class Worker(object):
                 result = numpy.vstack(integrated_result).T
 
         except Exception as err:
+            logger.debug("Backtrace", exc_info=True)
             err2 = ["error in integration do_2d: %s" % self.do_2D(),
                     str(err.__class__.__name__),
                     str(err),
@@ -350,9 +378,7 @@ class Worker(object):
                     "ai:",
                     str(self.ai),
                     "method:",
-                    kwarg.get("method")
-                    # str(self.ai._csr_integrator),
-                    # "csr size: %s" % self.ai._lut_integrator.size
+                    str(kwarg.get("method"))
                     ]
             logger.error("\n".join(err2))
             raise err
@@ -360,8 +386,13 @@ class Worker(object):
         if writer is not None:
             writer.write(integrated_result)
 
-        if self.output == "numpy":
-            return result
+        if self.output == "raw":
+            return integrated_result
+        elif self.output == "numpy":
+            if (variance is not None) and (error is not None):
+                return result, error
+            else:
+                return result
 
     def setSubdir(self, path):
         """
@@ -378,91 +409,117 @@ class Worker(object):
         else:
             self.extension = None
 
-    def setDarkcurrentFile(self, imagefile):
-        self.ai.set_darkfiles(imagefile)
+    def set_dark_current_file(self, imagefile):
+        self.ai.detector.set_darkcurrent(_reduce_images(imagefile))
         self.dark_current_image = imagefile
-
-    def setFlatfieldFile(self, imagefile):
-        self.ai.set_flatfiles(imagefile)
+    setDarkcurrentFile = set_dark_current_file
+    
+    def set_flat_field_file(self, imagefile):
+        self.ai.detector.set_flatfield(_reduce_images(imagefile))
         self.flat_field_image = imagefile
+    setFlatfieldFile = set_flat_field_file
+    
+    def set_config(self, config, consume_keys=False):
+        """
+        Configure the working from the dictionary.
 
-    def setJsonConfig(self, jsonconfig):
-        print("start config ...")
-        if os.path.isfile(jsonconfig):
-            with open(jsonconfig, "r") as f:
-                config = json.load(f)
-        else:
-            config = json.loads(jsonconfig)
-        if "poni" in config:
-            poni = config["poni"]
-            if poni and os.path.isfile(poni):
-                self.ai = AzimuthalIntegrator.sload(poni)
+        :param dict config: Key-value configuration
+        :param bool consume_keys: If true the keys from the dictionary will be
+            consumed when used.
+        """
+        if not consume_keys:
+            # Avoid to edit the input argument
+            config = dict(config)
 
-        detector = config.get("detector", "detector")
-        self.ai.detector = detector_factory(detector)
+        integration_config.normalize(config, inplace=True)
+        _init_ai(self.ai, config, consume_keys=True, read_maps=False)
 
-        if "wavelength" in config:
-            wavelength = config["wavelength"]
+        # Do it here before reading the AI to be able to catch the io
+        filename = config.pop("mask_file", "")
+        apply_process = config.pop("do_mask", True)
+        if filename and apply_process:
             try:
-                fwavelength = float(wavelength)
-            except ValueError:
-                logger.error("Unable to convert wavelength to float: %s", wavelength)
-            else:
-                if fwavelength <= 0 or fwavelength > 1e-6:
-                    logger.warning("Wavelength is in meter ... unlikely value %s", fwavelength)
-                self.ai.wavelength = fwavelength
-
-        splineFile = config.get("splineFile")
-        if splineFile and os.path.isfile(splineFile):
-            self.ai.detector.splineFile = splineFile
-        self.ai.pixel1 = float(config.get("pixel1", 1))
-        self.ai.pixel2 = float(config.get("pixel2", 1))
-        self.ai.dist = config.get("dist", 1)
-        self.ai.poni1 = config.get("poni1", 0)
-        self.ai.poni2 = config.get("poni2", 0)
-        self.ai.rot1 = config.get("rot1", 0)
-        self.ai.rot2 = config.get("rot2", 0)
-        self.ai.rot3 = config.get("rot3", 0)
-
-        if config.get("chi_discontinuity_at_0"):
-            self.ai.setChiDiscAtZero()
-        else:
-            self.ai.setChiDiscAtPi()
-
-        mask_file = config.get("mask_file")
-        do_mask = config.get("do_mask")
-        if mask_file and os.path.exists(mask_file) and do_mask:
-            try:
-                mask = fabio.open(mask_file).data
+                data = pyFAI.io.image.read_image_data(filename)
             except Exception as error:
-                logger.error("Unable to load mask file %s, error %s", mask_file, error)
+                logger.error("Unable to load mask file %s, error %s", filename, error)
             else:
-                self.ai.mask = mask
-                self.mask_image = os.path.abspath(mask_file)
+                self.ai.mask = data
 
-        self.ai.set_darkfiles([i.strip() for i in config.get("dark_current", "").split(",")
-                               if os.path.isfile(i.strip())])
-        self.ai.set_flatfiles([i.strip() for i in config.get("flat_field", "").split(",")
-                               if os.path.isfile(i.strip())])
-        self.dark_current_image = self.ai.darkfiles
-        self.flat_field_image = self.ai.flatfiles
-        if config.get("do_2D"):
-            self.nbpt_azim = int(config.get("nbpt_azim"))
+        # Do it here while we have to store metadata
+        filename = config.pop("dark_current", "")
+        apply_process = config.pop("do_dark", True)
+        if filename and apply_process:
+            filenames = _normalize_filenames(filename)
+            method = "mean"
+            data = _reduce_images(filenames, method=method)
+            self.ai.detector.set_darkcurrent(data)
+            self.dark_current_image = "%s(%s)" % (method, ",".join(filenames))
+
+        # Do it here while we have to store metadata
+        filename = config.pop("flat_field", "")
+        apply_process = config.pop("do_flat", True)
+        if filename and apply_process:
+            filenames = _normalize_filenames(filename)
+            method = "mean"
+            data = _reduce_images(filenames, method=method)
+            self.ai.detector.set_flatfield(data)
+            self.flat_field_image = "%s(%s)" % (method, ",".join(filenames))
+
+        # Uses it anyway in case do_2D is customed after the configuration
+        value = config.pop("nbpt_azim", None)
+        if value:
+            self.nbpt_azim = int(value)
         else:
             self.nbpt_azim = 1
-        if config.get("nbpt_rad"):
-            self.nbpt_rad = int(config["nbpt_rad"])
-        self.unit = units.to_unit(config.get("unit", units.TTH_DEG))
-        self.do_poisson = config.get("do_poisson")
-        if config.get("do_polarization"):
-            self.polarization_factor = config.get("polarization_factor")
+
+        reader = integration_config.ConfigurationReader(config)
+        self.method = reader.pop_method("csr")
+
+        if self.method.dim == 1:
+            self.nbpt_azim = 1
+
+        value = config.pop("nbpt_rad", None)
+        if value:
+            self.nbpt_rad = int(value)
+
+        value = config.pop("unit", units.TTH_DEG)
+        self.unit = units.to_unit(value)
+
+        value = config.pop("do_poisson", False)
+        self.do_poisson = bool(value)
+
+        value = config.pop("polarization_factor", None)
+        apply_value = config.pop("do_polarization", True)
+        if value and apply_value:
+            self.polarization_factor = value
         else:
             self.polarization_factor = None
 
-        if config.get("do_OpenCL"):
-            self.method = "csr_ocl"
-        else:
-            self.method = "csr"
+        value1 = config.pop("azimuth_range_min", None)
+        value2 = config.pop("azimuth_range_max", None)
+        apply_values = config.pop("do_azimuthal_range", True)
+        if apply_values and value1 is not None and value2 is not None:
+            self.azimuth_range = float(value1), float(value2)
+
+        value1 = config.pop("radial_range_min", None)
+        value2 = config.pop("radial_range_max", None)
+        apply_values = config.pop("do_radial_range", True)
+        if apply_values and value1 is not None and value2 is not None:
+            self.radial_range = float(value1), float(value2)
+
+        value = config.pop("do_solid_angle", True)
+        self.correct_solid_angle = bool(value)
+
+        self.dummy = config.pop("delta_dummy", None)
+        self.delta_dummy = config.pop("val_dummy", None)
+        apply_values = config.pop("do_dummy", True)
+        if not apply_values:
+            self.dummy, self.delta_dummy = None, None
+
+        self._normalization_factor = config.pop("normalization_factor", None)
+
+        if "monitor_name" in config:
+            logger.warning("Monitor name defined but unsupported by the worker.")
 
         logger.info(self.ai.__repr__())
         self.reset()
@@ -473,7 +530,6 @@ class Worker(object):
 
     def get_unit(self):
         return self._unit
-
     unit = property(get_unit, set_unit)
 
     def set_error_model(self, value):
@@ -483,7 +539,6 @@ class Worker(object):
             self.do_poisson = False
         else:
             raise RuntimeError("Unsupported error model '%s'" % value)
-
     def get_error_model(self):
         if self.do_poisson:
             return "poisson"
@@ -491,7 +546,10 @@ class Worker(object):
     error_model = property(get_error_model, set_error_model)
 
     def get_config(self):
-        """return configuration as a dictionary"""
+        """Returns the configuration as a dictionary.
+
+        FIXME: The returned dictionary is not exhaustive.
+        """
         config = {"unit": str(self.unit)}
         for key in ["dist", "poni1", "poni2", "rot1", "rot3", "rot2", "pixel1", "pixel2", "splineFile", "wavelength"]:
             try:
@@ -505,47 +563,41 @@ class Worker(object):
                 config[key] = self.__getattribute__(key)
             except:
                 pass
+        
+        for key in ["azimuth_range", "radial_range"]:
+            try:
+                value = self.__getattribute__(key)
+            except:
+                pass
+            else:
+                if value is not None:
+                    config["do_"+key] = True
+                    config[key+"_min"] = min(value)
+                    config[key+"_max"] = max(value)
+                else:
+                    config["do_"+key] = False
 
         return config
-#
-#    "poni" #path of the file
-#
-#    "chi_discontinuity_at_0"
-#    "do_mask"
-#    "do_dark"
-#    "do_azimuthal_range"
-#    "do_flat"
-#    "do_2D"
-#    "azimuth_range_min"
-#    "azimuth_range_max"
-#
-#    "polarization_factor"
-#    "nbpt_rad"
-#    "do_solid_angle"
-#    "do_radial_range"
-#    "do_poisson"
-#    "delta_dummy"
-#    "nbpt_azim"
-#    "flat_field"
-#    "radial_range_min"
-#    "dark_current"
-#    "do_polarization"
-#    "mask_file"
-#    "detector"
-#    "unit"
-#    "radial_range_max"
-#    "val_dummy"
-#    "do_dummy"
-#    "method"
-# }
 
     def get_json_config(self):
         """return configuration as a JSON string"""
-        pass  # TODO
+        return json.dumps(self.get_config(), indent=2)
 
+    def set_json_config(self, json_file):
+        if os.path.isfile(json_file):
+            with open(json_file, "r") as f:
+                config = json.load(f)
+        else:
+            config = json.loads(json_file)
+        self.set_config(config)
+    setJsonConfig = set_json_config
+    
     def save_config(self, filename=None):
+        """Save the configuration as a JSON file"""
         if not filename:
             filename = self.config_file
+        with open(filename, "w") as w:
+            w.write(self.get_json_config())
 
     def warmup(self, sync=False):
         """
@@ -564,12 +616,30 @@ class Worker(object):
     def get_normalization_factor(self):
         with self._sem:
             return self._normalization_factor
-
     def set_normalization_factor(self, value):
         with self._sem:
             self._normalization_factor = value
     normalization_factor = property(get_normalization_factor, set_normalization_factor)
 
+    def set_method(self, method="csr"):
+        "Set the integration method"
+        dim = 2 if self.do_2D() else 1
+        if method is None:
+            method = method_registry.Method(dim, "*", "*", "*", target=None)
+        elif isinstance(method, method_registry.Method):
+            method = method.fixed(dim=dim)
+        elif isinstance(method, six.string_types):
+            method = method_registry.Method.parsed(method)
+            method = method.fixed(dim=dim)
+        elif isinstance(method, (list, tuple)):
+            if len(method) != 3:
+                raise TypeError("Method size %s unsupported." % len(method))
+            split, algo, impl = method
+            method = method_registry.Method(dim, split, algo, impl, target=None)
+        else:
+            raise TypeError("Method type %s unsupported." % type(method))
+        return method
+
     __call__ = process
 
 
@@ -620,7 +690,8 @@ class PixelwiseWorker(object):
         self.empty = float(empty) if empty else 0.0
         self.dtype = numpy.dtype(dtype).type
 
-    def process(self, data, variance=None, normalization_factor=None):
+    def process(self, data, variance=None, normalization_factor=None,
+                use_cython=USE_CYTHON):
         """
         Process the data and apply a normalization factor
         :param data: input data
@@ -629,71 +700,33 @@ class PixelwiseWorker(object):
         :return: processed data, optionally with the assiciated error if variance is provided
         """
         propagate_error = (variance is not None)
-        if (preproc is not None) and USE_CYTHON:
-            temp_data = preproc(data,
-                                variance=variance,
-                                dark=self.dark,
-                                flat=self.flat,
-                                solidangle=self.solidangle,
-                                polarization=self.polarization,
-                                absorption=None,
-                                mask=self.mask,
-                                dummy=self.dummy,
-                                delta_dummy=self.delta_dummy,
-                                normalization_factor=normalization_factor,
-                                empty=self.empty,
-                                poissonian=0,
-                                dtype=self.dtype)
-            if propagate_error:
-                proc_data = temp_data[..., 0]
-                proc_variance = temp_data[..., 1]
-                proc_norm = temp_data[..., 2]
-                proc_data /= proc_norm
-                proc_error = numpy.sqrt(proc_variance) / proc_norm
-            else:
-                proc_data = temp_data
+        if use_cython:
+            method = preproc
         else:
-            if self.dummy is not None:
-                if self.delta_dummy is None:
-                    self.mask = numpy.logical_or((data == self.dummy), self.mask)
-                else:
-                    self.mask = numpy.logical_or(abs(data - self.dummy) <= self.delta_dummy,
-                                                 self.mask)
-                do_mask = True
-            else:
-                do_mask = (self.mask is not False)
-            # Explicitly make an copy !
-            proc_data = numpy.array(data, dtype=numpy.float32)
-            if self.dark is not None:
-                proc_data -= self.dark
-            if self.flat is not None:
-                proc_data /= self.flat
-            if self.solidangle is not None:
-                proc_data /= self.solidangle
-            if self.polarization is not None:
-                proc_data /= self.polarization
-            if normalization_factor is not None:
-                proc_data /= normalization_factor
-            if do_mask:
-                proc_data[self.mask] = self.dummy or self.empty
-
-            if variance is not None:
-                proc_error = numpy.sqrt(variance)
-
-                if self.flat is not None:
-                    proc_error /= self.flat
-                if self.solidangle is not None:
-                    proc_error /= self.solidangle
-                if self.polarization is not None:
-                    proc_error /= self.polarization
-                if normalization_factor is not None:
-                    proc_error /= normalization_factor
-                if do_mask:
-                    proc_error[self.mask] = self.dummy or self.empty
-
+            method = preproc_numpy
+        temp_data = method(data,
+                           variance=variance,
+                           dark=self.dark,
+                           flat=self.flat,
+                           solidangle=self.solidangle,
+                           polarization=self.polarization,
+                           absorption=None,
+                           mask=self.mask,
+                           dummy=self.dummy,
+                           delta_dummy=self.delta_dummy,
+                           normalization_factor=normalization_factor,
+                           empty=self.empty,
+                           poissonian=0,
+                           dtype=self.dtype)
         if propagate_error:
+            proc_data = temp_data[..., 0]
+            proc_variance = temp_data[..., 1]
+            proc_norm = temp_data[..., 2]
+            proc_data /= proc_norm
+            proc_error = numpy.sqrt(proc_variance) / proc_norm
             return proc_data, proc_error
         else:
+            proc_data = temp_data
             return proc_data
 
     __call__ = process
@@ -758,45 +791,19 @@ class DistortionWorker(object):
         :param variance: the variance associated to the data
         :param normalization: normalization factor
         :return: processed data
-        
-        TODO: manage variance in distortion correction
         """
-        if preproc is not None:
-            proc_data = preproc(data,
-                                dark=self.dark,
-                                flat=self.flat,
-                                solidangle=self.solidangle,
-                                polarization=self.polarization,
-                                absorption=None,
-                                mask=self.mask,
-                                dummy=self.dummy,
-                                delta_dummy=self.delta_dummy,
-                                normalization_factor=normalization_factor,
-                                empty=None)
-        else:
-            if self.dummy is not None:
-                if self.delta_dummy is None:
-                    self.mask = numpy.logical_or((data == self.dummy), self.mask)
-                else:
-                    self.mask = numpy.logical_or(abs(data - self.dummy) <= self.delta_dummy,
-                                                 self.mask)
-                do_mask = True
-            else:
-                do_mask = (self.mask is not False)
-            # Explicitly make an copy !
-            proc_data = numpy.array(data, dtype=numpy.float32)
-            if self.dark is not None:
-                proc_data -= self.dark
-            if self.flat is not None:
-                proc_data /= self.flat
-            if self.solidangle is not None:
-                proc_data /= self.solidangle
-            if self.polarization is not None:
-                proc_data /= self.polarization
-            if normalization_factor is not None:
-                proc_data /= normalization_factor
-            if do_mask:
-                proc_data[self.mask] = self.dummy or 0
+        proc_data = preproc(data,
+                            variance=variance,
+                            dark=self.dark,
+                            flat=self.flat,
+                            solidangle=self.solidangle,
+                            polarization=self.polarization,
+                            absorption=None,
+                            mask=self.mask,
+                            dummy=self.dummy,
+                            delta_dummy=self.delta_dummy,
+                            normalization_factor=normalization_factor,
+                            empty=None)
 
         if self.distortion is not None:
             return self.distortion.correct(proc_data, self.dummy, self.delta_dummy)
